----------------------------------------------------------------------------------
-- Company: UGA
-- Engineer: Cdric Meyer
-- 
-- Create Date: 08.11.2022 22:26:33
-- Module Name: AsservRB444_PI - Behavioral
-- Description: 
-- Description de haut niveau pour la boucle complete d'asservissement d'un moteur RB444
-- avec une rgulation PI
-- Instancie
--      - une base de temps (pour l'chantillonnage de la boucle et la mesure de la vitesse) 
--      - un composant de mesure de vitesse
--      - un composant de gnration des signaux de commande de pont en H (PWM + DIR)
--      - un correcteur PI
--      - un sommateur

-- Revision:
-- Revision 0.01 - File Created

----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_signed.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

entity AsservRB444_PI is
generic(inv_DIR:std_logic:='0'); -- parametre qui est utilis lors de l'instantiation du composant de gnration des signaux PWM+DIR pour inverser facilement le signal DIR
  Port (
        H_125MHz : in std_logic;
        cod_A, cod_B : in std_logic;
        V_consigne : in std_logic_vector (31 downto 0); -- doit tre compris entre -500 et +500 (la vitesse max 500 est trs rapide !)
        V_mesure : out std_logic_vector (31 downto 0);
        DIR, PWM : out std_logic
   );
end AsservRB444_PI;

architecture Behavioral of AsservRB444_PI is

component BaseDeTemps is
    Port ( H : in STD_LOGIC;
           T1us, T1ms, T10ms, T100ms, T1s, LED : out STD_LOGIC);
end component;

component vitesseCodeur IS 
generic(cptMax:integer:=20000);
port
	(
	A,B,H,T1us:IN std_logic;
	V:out std_logic_vector(31 downto 0)
	--V:out integer range -32767 to 32767:=0
	);
end component;

component GenePWM is 
generic(sens:std_logic:='0');-- pour inverser facilement la commande en fonction du sens de branchement du moteur
port
	(
	H:in std_logic;
	TON:in std_logic_vector(31 downto 0);-- Nombre de p�riodes d'horloge du temps haut du PWM (doit �tre compris entre -6250 et +6250)
	DIR:out std_logic:='0';
	PWM:out std_logic:='0'
	);
end component;

component CorrecteurPI is
generic(
    Kp:integer:=1000;
    KiTe:integer:=0
);
Port (
	H, Te : in std_logic;
    E : in std_logic_vector(31 downto 0);
    S : out std_logic_vector(31 downto 0)
);
end component;
signal T1us, T10ms : std_logic;
signal Vmes_i, erreur, V_commande, V_mesure_i : std_logic_vector(31 downto 0);
begin
U1:BaseDeTemps port map(H=>H_125MHz,T1us=>T1us,T10ms=>T10ms);
U2:vitesseCodeur generic map(cptMax=>20000) port map(A=>cod_A, B=>cod_B,H=>H_125MHz,T1us=>T1us,V=>V_mesure_i);
U3:GenePWM generic map(sens=>inv_DIR) port map(H=>H_125MHz, TON=>V_commande, DIR=>DIR, PWM=>PWM);
U4:CorrecteurPI generic map(Kp=>200,KiTe=>2500) port map(H=>H_125MHz, Te=>T10ms, E=>erreur, S=>V_commande);

process(H_125MHz)
begin
if(rising_edge(H_125MHz)) then
    erreur<=V_consigne - V_mesure_i;
end if;
end process;
V_mesure<=V_mesure_i;
end Behavioral;
