
library ieee;
use ieee.STD_LOGIC_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_signed.all;
use ieee.math_real.all;


entity CorrecteurPI is
generic(
    Kp:integer:=1000;
    KiTe:integer:=0
);
Port (
	H, Te : in std_logic;
    E : in std_logic_vector(31 downto 0);
    S : out std_logic_vector(31 downto 0)
);
end CorrecteurPI;

architecture Behavioral of CorrecteurPI is
signal E_n_1,S_n,S_n_1 : std_logic_vector(31 downto 0):=(others=>'0');
signal A1,A2:integer range -2**31 to 2**31-1;
constant S_max : std_logic_vector(31 downto 0):=std_logic_vector(to_signed( 10000 ,32));

begin

process(H)
begin
	if(rising_edge(H)) then
		if(Te = '1') then
			E_n_1<=E;
			if(S_n >S_max)then S_n_1<=S_max;
			elsif(S_n <-S_max)then S_n_1<=-S_max;
			else S_n_1<=S_n;
			end if;
		end if;
	end if;			
end process;


A1 <= to_integer(signed(E))*( (Kp+KiTe)) /1024;
A2 <= to_integer(signed(E_n_1))*( (Kp) )/1024;
S_n <= S_n_1 + std_logic_vector(to_signed( A1-A2 ,32));
S <= S_n;
end Behavioral;
