library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_signed.all;
use ieee.numeric_std.all;

-- F_PWM = 20kHZ => T_PWM = 50us
-- F_Clk = 125MHz => T_Clk = 8ns 
-- D'ou un rapport de 6250 entre les deux
-- Il faut donc 14 bits pour reprsenter cette valeur en sign
entity GenePWM is 
generic(sens:std_logic:='0');-- pour inverser facilement la commande en fonction du sens de branchement du moteur
port
	(
	H:in std_logic;
	inv_DIR : in std_logic; -- signal permettant d'inverser la sortie DIR (si inv_DIR=1)
	TON:in std_logic_vector(31 downto 0);-- Nombre de p�riodes d'horloge du temps haut du PWM (doit �tre compris entre -6250 et +6250)
	DIR:out std_logic:='0';
	PWM:out std_logic:='0'

	);
END ;
ARCHITECTURE a_GenePWM OF GenePWM IS
	signal cpt : integer range 0 to 6250;
	signal PWM_i : std_logic:='0';
	signal DIR_i : std_logic;
BEGIN
	PROCESS (H)
	BEGIN
		if (H'event and H='1') then
			if (cpt = 6249) then 
				cpt<=0;
				if TON/=0 then PWM_i<='1'; end if;
			elsif (cpt = (abs TON)-1) then
				cpt<=cpt+1;
				PWM_i<='0';			 	
			else
			 	cpt<=cpt+1;
			end if;
		end IF;
	END PROCESS;
	PWM<='0' when TON=0 else PWM_i;
	DIR_i<='0' when ( (TON>0 and sens='0') or (TON<=0 and sens='1') ) else '1';
	DIR<= DIR_i when inv_DIR = '0' else not DIR_i;
	
END;

